@@ -31,6 +31,7 @@
 #include "debug.h"
 #include "prefs.h"
 #include "request.h"
+#include "pidgintooltip.h"
 
 #include <string.h>
 
@@ -41,7 +42,15 @@ static void plugin_toggled_stage_two(PurplePlugin *plug, GtkTreeModel *model,
 
 static GtkWidget *expander = NULL;
 static GtkWidget *plugin_dialog = NULL;
-static GtkWidget *plugin_details = NULL;
+
+static GtkLabel *plugin_name = NULL;
+static GtkTextBuffer *plugin_desc = NULL;
+static GtkLabel *plugin_error = NULL;
+static GtkLabel *plugin_author = NULL;
+static GtkLabel *plugin_website = NULL;
+static gchar *plugin_website_uri = NULL;
+static GtkLabel *plugin_filename = NULL;
+
 static GtkWidget *pref_button = NULL;
 static GHashTable *plugin_pref_dialogs = NULL;
 
@@ -80,9 +89,7 @@ pidgin_plugin_get_config_frame(PurplePlugin *plugin)
 
 		config = pidgin_plugin_pref_create_frame(frame);
 
-		/* XXX According to bug #1407047 this broke saving pluging preferences, I'll look at fixing it correctly later.
-		purple_plugin_pref_frame_destroy(frame);
-		*/
+		plugin->info->prefs_info->frame = frame;
 	}
 
 	return config;
@@ -128,9 +135,15 @@ update_plugin_list(void *data)
 
 		gtk_list_store_append (ls, &iter);
 
-		name = g_markup_escape_text(plug->info->name ? _(plug->info->name) : g_basename(plug->path), -1);
-		version = g_markup_escape_text(plug->info->version, -1);
-		summary = g_markup_escape_text(_(plug->info->summary), -1);
+		if (plug->info->name) {
+			name = g_markup_escape_text(_(plug->info->name), -1);
+		} else {
+			char *tmp = g_path_get_basename(plug->path);
+			name = g_markup_escape_text(tmp, -1);
+			g_free(tmp);
+		}
+		version = g_markup_escape_text(purple_plugin_get_version(plug), -1);
+		summary = g_markup_escape_text(purple_plugin_get_summary(plug), -1);
 
 		desc = g_strdup_printf("<b>%s</b> %s\n%s", name,
 				       version,
@@ -211,6 +224,12 @@ static void pref_dialog_response_cb(GtkWidget *d, int response, PurplePlugin *pl
 			plugin_pref_dialogs = NULL;
 		}
 		gtk_widget_destroy(d);
+
+		if (plug->info->prefs_info && plug->info->prefs_info->frame) {
+			purple_plugin_pref_frame_destroy(plug->info->prefs_info->frame);
+			plug->info->prefs_info->frame = NULL;
+		}
+
 		break;
 	}
 }
@@ -272,7 +291,7 @@ static void plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer dat
 				PurplePlugin *dep_plugin = purple_plugins_find_with_id(dep_name);
 				g_return_if_fail(dep_plugin != NULL);
 
-				g_string_append_printf(tmp, "\n\t%s\n", _(dep_plugin->info->name));
+				g_string_append_printf(tmp, "\n\t%s\n", purple_plugin_get_name(dep_plugin));
 			}
 
 			cb_data = g_new(gpointer, 3);
@@ -280,11 +299,11 @@ static void plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer dat
 			cb_data[1] = model;
 			cb_data[2] = iter;
 
-			purple_request_action_with_hint(plugin_dialog, NULL,
+			purple_request_action(plugin_dialog, NULL,
 			                    _("Multiple plugins will be unloaded."),
 			                    tmp->str, 0,
-			                    NULL, NULL, NULL,
-			                    PURPLE_REQUEST_UI_HINT_PLUGINMGR, cb_data, 2,
+								NULL, NULL, NULL,
+								cb_data, 2,
 			                    _("Unload Plugins"), G_CALLBACK(plugin_unload_confirm_cb),
 			                    _("Cancel"), g_free);
 			g_string_free(tmp, TRUE);
@@ -296,9 +315,6 @@ static void plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer dat
 
 static void plugin_toggled_stage_two(PurplePlugin *plug, GtkTreeModel *model, GtkTreeIter *iter, gboolean unload)
 {
-	gchar *name = NULL;
-	gchar *description = NULL;
-
 	if (unload)
 	{
 		pidgin_set_cursor(plugin_dialog, GDK_WATCH);
@@ -332,30 +348,30 @@ static void plugin_toggled_stage_two(PurplePlugin *plug, GtkTreeModel *model, Gt
 		 || (plug->info->prefs_info
 			&& plug->info->prefs_info->get_plugin_pref_frame)));
 
-	name = g_markup_escape_text(_(plug->info->name), -1);
-	description = g_markup_escape_text(_(plug->info->description), -1);
+	if (plug->error != NULL)
+	{
+		gchar *name = g_markup_escape_text(purple_plugin_get_name(plug), -1);
 
-	if (plug->error != NULL) {
 		gchar *error = g_markup_escape_text(plug->error, -1);
-		gchar *desc;
-		gchar *text = g_strdup_printf(
-				   "<span size=\"larger\">%s %s</span>\n\n"
-				   "<span weight=\"bold\" color=\"red\">%s</span>\n\n"
-				   "%s",
-				   name, plug->info->version, error, description);
-		desc = g_strdup_printf("<b>%s</b> %s\n<span weight=\"bold\" color=\"red\"%s</span>",
-			       plug->info->name, plug->info->version, error);
-		gtk_list_store_set(GTK_LIST_STORE (model), iter,
-				   1, desc,
-				   -1);
-		g_free(desc);
-		g_free(error);
-		gtk_label_set_markup(GTK_LABEL(plugin_details), text);
-		g_free(text);
-	}
-	g_free(name);
-	g_free(description);
+		gchar *text;
 
+		text = g_strdup_printf(
+			"<b>%s</b> %s\n<span weight=\"bold\" color=\"red\"%s</span>",
+			purple_plugin_get_name(plug), purple_plugin_get_version(plug), error);
+		gtk_list_store_set(GTK_LIST_STORE (model), iter,
+				   1, text,
+				   -1);
+		g_free(text);
+
+		text = g_strdup_printf(
+			"<span weight=\"bold\" color=\"red\">%s</span>",
+			error);
+		gtk_label_set_markup(plugin_error, text);
+		g_free(text);
+
+		g_free(error);
+		g_free(name);
+	}
 
 	gtk_list_store_set(GTK_LIST_STORE (model), iter,
 	                   0, purple_plugin_is_loaded(plug),
@@ -382,15 +398,13 @@ static gboolean ensure_plugin_visible(void *data)
 
 static void prefs_plugin_sel (GtkTreeSelection *sel, GtkTreeModel *model)
 {
-	gchar *buf, *pname, *pdesc, *pauth, *pweb;
+	gchar *buf, *tmp, *name, *version;
 	GtkTreeIter  iter;
 	GValue val;
 	PurplePlugin *plug;
 
 	if (!gtk_tree_selection_get_selected (sel, &model, &iter))
 	{
-		/* Clear the old plugin details */
-		gtk_label_set_markup(GTK_LABEL(plugin_details), "");
 		gtk_widget_set_sensitive(pref_button, FALSE);
 
 		/* Collapse and disable the expander widget */
@@ -406,32 +420,53 @@ static void prefs_plugin_sel (GtkTreeSelection *sel, GtkTreeModel *model)
 	gtk_tree_model_get_value (model, &iter, 2, &val);
 	plug = g_value_get_pointer(&val);
 
-	pname = g_markup_escape_text(_(plug->info->name), -1);
-	pdesc = (plug->info->description) ?
-			g_markup_escape_text(_(plug->info->description), -1) : NULL;
-	pauth = (plug->info->author) ?
-			g_markup_escape_text(_(plug->info->author), -1) : NULL;
-	pweb = (plug->info->homepage) ?
-		   g_markup_escape_text(_(plug->info->homepage), -1) : NULL;
+	name = g_markup_escape_text(purple_plugin_get_name(plug), -1);
+	version = g_markup_escape_text(purple_plugin_get_version(plug), -1);
 	buf = g_strdup_printf(
-		   _("%s%s"
-		     "<span weight=\"bold\">Written by:</span>\t%s\n"
-		     "<span weight=\"bold\">Website:</span>\t\t%s\n"
-		     "<span weight=\"bold\">Filename:</span>\t\t%s"),
-		   pdesc ? pdesc : "", pdesc ? "\n\n" : "",
-		   pauth ? pauth : "", pweb ? pweb : "", plug->path);
+		"<span size=\"larger\" weight=\"bold\">%s</span> "
+		"<span size=\"smaller\">%s</span>",
+		name, version);
+	gtk_label_set_markup(plugin_name, buf);
+	g_free(name);
+	g_free(version);
+	g_free(buf);
 
-	if (plug->error != NULL)
+	gtk_text_buffer_set_text(plugin_desc, purple_plugin_get_description(plug), -1);
+	gtk_label_set_text(plugin_author, purple_plugin_get_author(plug));
+	gtk_label_set_text(plugin_filename, plug->path);
+
+	g_free(plugin_website_uri);
+	plugin_website_uri = g_strdup(purple_plugin_get_homepage(plug));
+	if (plugin_website_uri)
 	{
-		char *tmp = g_strdup_printf(
-			_("%s\n"
-			  "<span foreground=\"#ff0000\" weight=\"bold\">"
+		tmp = g_markup_escape_text(plugin_website_uri, -1);
+		buf = g_strdup_printf("<span underline=\"single\" "
+			"foreground=\"blue\">%s</span>", tmp);
+		gtk_label_set_markup(plugin_website, buf);
+		g_free(tmp);
+		g_free(buf);
+	}
+	else
+	{
+		gtk_label_set_text(plugin_website, NULL);
+	}
+
+	if (plug->error == NULL)
+	{
+		gtk_label_set_text(plugin_error, NULL);
+	}
+	else
+	{
+		tmp = g_markup_escape_text(plug->error, -1);
+		buf = g_strdup_printf(
+			_("<span foreground=\"red\" weight=\"bold\">"
 			  "Error: %s\n"
 			  "Check the plugin website for an update."
 			  "</span>"),
-			buf, plug->error);
+			tmp);
+		gtk_label_set_markup(plugin_error, buf);
 		g_free(buf);
-		buf = tmp;
+		g_free(tmp);
 	}
 
 	gtk_widget_set_sensitive(pref_button,
@@ -441,18 +476,10 @@ static void prefs_plugin_sel (GtkTreeSelection *sel, GtkTreeModel *model)
 		 || (plug->info->prefs_info
 			&& plug->info->prefs_info->get_plugin_pref_frame)));
 
-	gtk_label_set_markup(GTK_LABEL(plugin_details), buf);
-
 	/* Make sure the selected plugin is still visible */
 	g_idle_add(ensure_plugin_visible, sel);
 
-
 	g_value_unset(&val);
-	g_free(buf);
-	g_free(pname);
-	g_free(pdesc);
-	g_free(pauth);
-	g_free(pweb);
 }
 
 static void plugin_dialog_response_cb(GtkWidget *d, int response, GtkTreeSelection *sel)
@@ -500,8 +527,10 @@ static void plugin_dialog_response_cb(GtkWidget *d, int response, GtkTreeSelecti
 		g_hash_table_insert(plugin_pref_dialogs, plug, dialog);
 
 		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(pref_dialog_response_cb), plug);
-		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
-			_pidgin_make_container_scrollable(box, dialog));
+#ifdef USE_HILDON
+		box = _pidgin_make_container_scrollable(box, -1, -1);
+#endif /* USE_HILDON */
+		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), box);
 		gtk_window_set_role(GTK_WINDOW(dialog), "plugin_config");
 		gtk_window_set_title(GTK_WINDOW(dialog), _(purple_plugin_get_name(plug)));
 		gtk_widget_show_all(dialog);
@@ -532,12 +561,157 @@ show_plugin_prefs_cb(GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *co
 	plugin_dialog_response_cb(dialog, PIDGIN_RESPONSE_CONFIGURE, sel);
 }
 
-void pidgin_plugin_dialog_show()
+static gboolean
+pidgin_plugins_paint_tooltip(GtkWidget *tipwindow, gpointer data)
 {
-	pidgin_plugin_dialog_show_with_parent(NULL);
+	PangoLayout *layout = g_object_get_data(G_OBJECT(tipwindow), "tooltip-plugin");
+	gtk_paint_layout(tipwindow->style, tipwindow->window, GTK_STATE_NORMAL, FALSE,
+			NULL, tipwindow, "tooltip",
+			6, 6, layout);
+	return TRUE;
 }
 
-void pidgin_plugin_dialog_show_with_parent(GtkWindow *parent)
+static gboolean
+pidgin_plugins_create_tooltip(GtkWidget *tipwindow, GtkTreePath *path,
+		gpointer data, int *w, int *h)
+{
+	GtkTreeIter iter;
+	GtkTreeView *treeview = GTK_TREE_VIEW(data);
+	PurplePlugin *plugin = NULL;
+	GtkTreeModel *model = gtk_tree_view_get_model(treeview);
+	PangoLayout *layout;
+	int width, height;
+	char *markup, *name, *desc, *author;
+
+	if (!gtk_tree_model_get_iter(model, &iter, path))
+		return FALSE;
+
+	gtk_tree_model_get(model, &iter, 2, &plugin, -1);
+
+	markup = g_strdup_printf("<span size='x-large' weight='bold'>%s</span>\n<b>%s:</b> %s\n<b>%s:</b> %s",
+			name = g_markup_escape_text(purple_plugin_get_name(plugin), -1),
+			_("Description"), desc = g_markup_escape_text(purple_plugin_get_description(plugin), -1),
+			_("Author"), author = g_markup_escape_text(purple_plugin_get_author(plugin), -1));
+
+	layout = gtk_widget_create_pango_layout(tipwindow, NULL);
+	pango_layout_set_markup(layout, markup, -1);
+	pango_layout_set_wrap(layout, PANGO_WRAP_WORD);
+	pango_layout_set_width(layout, 600000);
+	pango_layout_get_size(layout, &width, &height);
+	g_object_set_data_full(G_OBJECT(tipwindow), "tooltip-plugin", layout, g_object_unref);
+
+	if (w)
+		*w = PANGO_PIXELS(width) + 12;
+	if (h)
+		*h = PANGO_PIXELS(height) + 12;
+
+	g_free(markup);
+	g_free(name);
+	g_free(desc);
+	g_free(author);
+
+	return TRUE;
+}
+
+static gboolean
+website_button_motion_cb(GtkWidget *button, GdkEventCrossing *event,
+                          gpointer unused)
+{
+	if (plugin_website_uri) {
+		pidgin_set_cursor(button, GDK_HAND2);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static gboolean
+website_button_clicked_cb(GtkButton *button, GdkEventButton *event,
+                          gpointer unused)
+{
+	if (plugin_website_uri) {
+		purple_notify_uri(NULL, plugin_website_uri);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static GtkWidget *
+create_details()
+{
+	GtkBox *vbox = GTK_BOX(gtk_vbox_new(FALSE, 3));
+	GtkSizeGroup *sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	GtkWidget *label, *view, *website_button;
+
+	plugin_name = GTK_LABEL(gtk_label_new(NULL));
+	gtk_misc_set_alignment(GTK_MISC(plugin_name), 0, 0);
+	gtk_label_set_line_wrap(plugin_name, FALSE);
+	gtk_label_set_selectable(plugin_name, TRUE);
+	gtk_box_pack_start(vbox, GTK_WIDGET(plugin_name), FALSE, FALSE, 0);
+
+	view = gtk_text_view_new();
+	plugin_desc = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+	g_object_set(view, "wrap-mode", GTK_WRAP_WORD,
+	                   "editable",  FALSE,
+	                   "left-margin",  PIDGIN_HIG_CAT_SPACE,
+	                   "right-margin", PIDGIN_HIG_CAT_SPACE,
+	                   NULL);
+	gtk_box_pack_start(vbox, view, TRUE, TRUE, 0);
+
+	plugin_error = GTK_LABEL(gtk_label_new(NULL));
+	gtk_misc_set_alignment(GTK_MISC(plugin_error), 0, 0);
+	gtk_label_set_line_wrap(plugin_error, FALSE);
+	gtk_label_set_selectable(plugin_error, TRUE);
+	gtk_box_pack_start(vbox, GTK_WIDGET(plugin_error), FALSE, FALSE, 0);
+
+	plugin_author = GTK_LABEL(gtk_label_new(NULL));
+	gtk_label_set_line_wrap(plugin_author, FALSE);
+	gtk_misc_set_alignment(GTK_MISC(plugin_author), 0, 0);
+	gtk_label_set_selectable(plugin_author, TRUE);
+	pidgin_add_widget_to_vbox(vbox, "", sg,
+		GTK_WIDGET(plugin_author), TRUE, &label);
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Written by:</b>"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+
+	website_button = gtk_event_box_new();
+#if GTK_CHECK_VERSION(2,4,0)
+	gtk_event_box_set_visible_window(GTK_EVENT_BOX(website_button), FALSE);
+#endif
+
+	plugin_website = GTK_LABEL(gtk_label_new(NULL));
+#if GTK_CHECK_VERSION(2,6,0)
+	g_object_set(G_OBJECT(plugin_website),
+		"ellipsize", PANGO_ELLIPSIZE_MIDDLE, NULL);
+#endif
+	gtk_misc_set_alignment(GTK_MISC(plugin_website), 0, 0);
+	gtk_container_add(GTK_CONTAINER(website_button),
+		GTK_WIDGET(plugin_website));
+	g_signal_connect(website_button, "button-release-event",
+		G_CALLBACK(website_button_clicked_cb), NULL);
+	g_signal_connect(website_button, "enter-notify-event",
+		G_CALLBACK(website_button_motion_cb), NULL);
+	g_signal_connect(website_button, "leave-notify-event",
+		G_CALLBACK(pidgin_clear_cursor), NULL);
+
+	pidgin_add_widget_to_vbox(vbox, "", sg, website_button, TRUE, &label);
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Web site:</b>"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+
+	plugin_filename = GTK_LABEL(gtk_label_new(NULL));
+	gtk_label_set_line_wrap(plugin_filename, FALSE);
+	gtk_misc_set_alignment(GTK_MISC(plugin_filename), 0, 0);
+	gtk_label_set_selectable(plugin_filename, TRUE);
+	pidgin_add_widget_to_vbox(vbox, "", sg,
+		GTK_WIDGET(plugin_filename), TRUE, &label);
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Filename:</b>"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+
+	g_object_unref(sg);
+
+	return GTK_WIDGET(vbox);
+}
+
+
+void pidgin_plugin_dialog_show()
 {
 	GtkWidget *sw;
 	GtkWidget *event_view;
@@ -547,8 +721,6 @@ void pidgin_plugin_dialog_show_with_parent(GtkWindow *parent)
 	GtkTreeSelection *sel;
 
 	if (plugin_dialog != NULL) {
-		if (parent)
-			gtk_window_set_transient_for(GTK_WINDOW(plugin_dialog), parent);
 		gtk_window_present(GTK_WINDOW(plugin_dialog));
 		return;
 	}
@@ -557,8 +729,6 @@ void pidgin_plugin_dialog_show_with_parent(GtkWindow *parent)
 						    NULL,
 						    GTK_DIALOG_NO_SEPARATOR,
 						    NULL);
-	if (parent)
-		gtk_window_set_transient_for(GTK_WINDOW(plugin_dialog), parent);
 	pref_button = gtk_dialog_add_button(GTK_DIALOG(plugin_dialog),
 						_("Configure Pl_ugin"), PIDGIN_RESPONSE_CONFIGURE);
 	gtk_dialog_add_button(GTK_DIALOG(plugin_dialog),
@@ -579,9 +749,11 @@ void pidgin_plugin_dialog_show_with_parent(GtkWindow *parent)
 	update_plugin_list(ls);
 
 	event_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ls));
+
+#ifdef USE_HILDON
 	if (g_object_class_find_property(g_type_class_peek(GTK_TYPE_TREE_VIEW), "allow-checkbox-mode"))
 		g_object_set(G_OBJECT(event_view), "allow-checkbox-mode", FALSE, NULL);
-
+#endif /* USE_HILDON */
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(event_view), TRUE);
 
 	g_signal_connect(G_OBJECT(event_view), "row-activated",
@@ -625,16 +797,28 @@ void pidgin_plugin_dialog_show_with_parent(GtkWindow *parent)
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(event_view),
 				pidgin_tree_view_search_equal_func, NULL, NULL);
 
+	pidgin_tooltip_setup_for_treeview(event_view, event_view,
+			pidgin_plugins_create_tooltip,
+			pidgin_plugins_paint_tooltip);
+
+
 	expander = gtk_expander_new(_("<b>Plugin Details</b>"));
 	gtk_expander_set_use_markup(GTK_EXPANDER(expander), TRUE);
-	plugin_details = gtk_label_new(NULL);
-	gtk_label_set_line_wrap(GTK_LABEL(plugin_details), TRUE);
-	gtk_container_add(GTK_CONTAINER(expander), _pidgin_make_container_scrollable(plugin_details, NULL));
 	gtk_widget_set_sensitive(expander, FALSE);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(plugin_dialog)->vbox), expander, FALSE, FALSE, 0);
+#ifdef USE_HILDON
+	gtk_container_add(GTK_CONTAINER(expander), _pidgin_make_container_scrollable(create_details(), -1, -1));
+#else /* !USE_HILDON */
+	gtk_container_add(GTK_CONTAINER(expander), create_details());
+#endif /* USE_HILDON */
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(plugin_dialog)->vbox), expander,
+		FALSE, FALSE, 0);
+
 
 	g_signal_connect (G_OBJECT (sel), "changed", G_CALLBACK (prefs_plugin_sel), NULL);
 	g_signal_connect(G_OBJECT(plugin_dialog), "response", G_CALLBACK(plugin_dialog_response_cb), sel);
-	gtk_window_set_default_size(GTK_WINDOW(plugin_dialog), 430, 430);
+	gtk_window_set_default_size(GTK_WINDOW(plugin_dialog), 430, 530);
+
+	pidgin_auto_parent_window(GTK_WINDOW(plugin_dialog));
+
 	gtk_widget_show_all(plugin_dialog);
 }
