@@ -1740,6 +1740,7 @@ static void load_conf(void)
 			"BAD wroking\nGOOD working\n"
 			"BAD wtih\nGOOD with\n"
 			"BAD wuould\nGOOD would\n"
+			"BAD wud\nGOOD would\n"
 			"BAD wut\nGOOD what\n"
 			"BAD wya\nGOOD way\n"
 			"BAD y\nGOOD why\n"
@@ -2195,8 +2196,10 @@ get_config_frame(PurplePlugin *plugin)
 	gtk_widget_show(win);
 
 	tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(model));
+#ifdef USE_HILDON
 	if (g_object_class_find_property(g_type_class_peek(GTK_TYPE_TREE_VIEW), "allow-checkbox-mode"))
 		g_object_set(G_OBJECT(tree), "allow-checkbox-mode", FALSE, NULL);
+#endif /* USE_HILDON */
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
 	gtk_widget_set_size_request(tree, -1, 200);
 
